﻿@publishTestResults @adoSpecific
Feature: Publish TestRunTrx Test Results

Rule: The test results can be loaded from an existing Test Run containing a TRX attachment

@tc:1175
Scenario: A Test Run for Azure DevOps is re-published for Test Cases
	Given there is a remote server project prepared for publishing test results
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		@tc:[id-of-test-case-1]
		Scenario: Passing scenario
			When the scenario pass

		@tc:[id-of-test-case-2]
		Scenario: Failing scenario
			When the scenario fail

		@tc:[id-of-test-case-3]
		Scenario Outline: Mixed results
			When the scenario <result>
		Examples:
			| result |
			| pass   |
			| fail   |
		"""
	And there is a test result file "local-result.trx" with
		| name              | outcome | 
		| PassingScenario   | Passed  | 
		| FailingScenario   | Failed  |
		| MixedResults_Pass | Passed  |
		| MixedResults_Fail | Failed  |
	And the test result file has been published to Azure DevOps
	And the test result publishing is configured with
		| setting    | value            |
		| fileFormat | testRunTRX       |
		| source     | [id-of-test-run] |
	When the test result is published successfully
	Then the there should be a Test Run registered with test results
		| test case ID        | outcome | 
		| [id-of-test-case-1] | Passed  | 
		| [id-of-test-case-2] | Failed  | 
		| [id-of-test-case-3] | Failed  | 
	And the Test Run should contain the following attachments
		| file name        |
		| local-result.trx |

@tc:1177
@notSupported:tfs2017
Scenario: The test result attachments of Test Run are also re-published
	Given there is a remote server project prepared for publishing test results
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Passing scenario
			When the scenario pass
		"""
	And there is a test result file with
		| name             | outcome | result files                     |
		| Passing scenario | Passed  | MYMACHINE\\sample_attachment.png |
	And the test result file has been published to Azure DevOps
	And the test result publishing is configured with
		| setting    | value            |
		| fileFormat | testRunTRX       |
		| source     | [id-of-test-run] |
	When the test result is published successfully
	Then the there should be a Test Run registered with test results
		| test case ID      | outcome | attachments                                       |
		| [id-of-test-case] | Passed  | sample_attachment.png,Standard_Console_Output.log |


Rule: The environment variable VSTEST_TESTRUNID should be used as Test Run ID if not specified

The VSTest task sets the VSTEST_TESTRUNID environment variable witht he published Test Run ID

@tc:1176
@bypass-integration
Scenario: The Test Run published by VSTest is re-published
	Given there is a remote server project prepared for publishing test results
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Passing scenario
			When the scenario pass
		"""
	And there is a test result file with
		| name             | outcome |
		| Passing scenario | Passed  |
	And the test result file has been published to Azure DevOps
	And the environment variable "VSTEST_TESTRUNID" is set to "[id-of-test-run]"
	And the test result publishing is configured with
		| setting    | value      |
		| fileFormat | testRunTRX |
	When the test result is published successfully
	Then the there should be a Test Run registered with test results
		| test case ID      | outcome |
		| [id-of-test-case] | Passed  |

Rule: When both the source Test Run and the new Test Run is associated to a pipeline, disable that so that the results are not displayed in the pipeline result duplicated

The behavior can be switched off by setting 'toolSettings/DoNotDisableSourceTestRun' to 'true'.

@tc:1178
Scenario: The source Test Run was associated to the build
	Given there is a remote server project prepared for publishing test results
	And the synchronizer is configured to enable automation
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Passing scenario
			When the scenario pass
		"""
	And there is a test result file with
		| name             | outcome |
		| Passing scenario | Passed  |
	And the test result file has been published to Azure DevOps to build "[existing-build-id]"
	And the test result publishing is configured with
		| setting    | value               |
		| fileFormat | testRunTRX          |
		| source     | [id-of-test-run]    |
		| buildId    | [existing-build-id] |
	When the test result is published successfully
	Then the there should be a Test Run registered with 
		| setting        | value               |
		| Build Id       | [existing-build-id] |
	And the source Test Run is disabled by changing the state to NotStarted
