﻿@pull @adoSpecific
Feature: Pull - Create new test cases
	
New test cases that are in a test suite can be synchronized back to new feature files

Rule: Unlinked Test Cases in the remote scope (Test Suite) are pulled as a new scenario in a new feature file

@tc:202
Scenario: Create feature file from new test case
	Given there is an Azure DevOps project with an empty Test Suite 'MySuite'
	And the synchronizer is configured to enable pull
	And the synchronizer is configured to add Test Cases to Test Suite 'MySuite'
	And the synchronizer is configured to create local scenarios for new Test Cases
	And there is a new Test Case in Test Suite 'MySuite' as
		| field | value                                         |
		| title | Scenario: Sample scenario                     |
		| steps | Given there is something; When I do something |
	When the local repository is synchronized with pull
	Then the local repository should contain a feature file '[id-of-test-case].feature' as
		"""
		Feature: [id-of-test-case]

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			Given there is something
			When I do something
		"""
	And the feature file should be included in the local scope

@tc:203
Scenario: Create feature file from new parametrized test case
	Given there is an Azure DevOps project with an empty Test Suite 'MySuite'
	And the synchronizer is configured to enable pull
	And the synchronizer is configured to add Test Cases to Test Suite 'MySuite'
	And the synchronizer is configured to create local scenarios for new Test Cases
	And there is a new Test Case in Test Suite 'MySuite' as
		| field | value                                               |
		| title | Scenario Outline: Sample scenario outline           |
		| steps | Given there is [[someone]]; When I do [[something]] |
	And the Test Case parameter data is updated to
		| someone |something | 
		| Tarzan  |one       | 
		| Thomas  |two       | 
	When the local repository is synchronized with pull
	Then the local repository should contain a feature file '[id-of-test-case].feature' as
		"""
		Feature: [id-of-test-case]

		@tc:[id-of-test-case]
		Scenario Outline: Sample scenario outline
			Given there is <someone>
			When I do <something>
		Examples: 
			| someone |something | 
			| Tarzan  |one       | 
			| Thomas  |two       | 
		"""
	And the feature file should be included in the local scope

@tc:204
Scenario: Create feature file from new test case without title prefix
	Given there is an Azure DevOps project with an empty Test Suite 'MySuite'
	And the synchronizer is configured to enable pull
	And the synchronizer is configured to add Test Cases to Test Suite 'MySuite'
	And the synchronizer is configured to create local scenarios for new Test Cases
	And there is a new Test Case in Test Suite 'MySuite' as
		| field | value                                         |
		| title | Sample scenario                               |
		| steps | Given there is something; When I do something |
	When the local repository is synchronized with pull
	Then the local repository should contain a feature file '[id-of-test-case].feature' as
		"""
		Feature: [id-of-test-case]

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			Given there is something
			When I do something
		"""
	And the feature file should be included in the local scope

Rule: Create-only mode: Pull can be configured to only pull unlinked Test Cases (do not change existing scenarios)

@tc:632
Scenario: Only new Test Cases are pulled
	Given there is an Azure DevOps project with an empty Test Suite 'MySuite'
	And the synchronizer is configured to add Test Cases to Test Suite 'MySuite'
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file 'Existing.feature'
		"""
		Feature: Existing feature
		@tc:[id-of-test-case] @mytag
		Scenario: Sample scenario
			When I do something
		"""
	And the Test Case tags have been updated to "mytag, othertag"
	And there is a new Test Case in Test Suite 'MySuite' as
		| field | value                                         |
		| title | Scenario: Sample scenario                     |
	When the local repository is synchronized with pull in create-only mode
	Then the feature file in the local repository should not change
	But the local repository should contain a feature file '[id-of-test-case].feature'

