﻿@pull
Feature: Pull

Pull changes from Azure DevOps server to the local repository.

Rule: Changes can be pulled

@tc:207
Scenario: Pull a normal test case to a scenario
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Scenario: Sample scenario
			When something happens with someone
		"""
	When the Test Case title should be updated to 'Scenario: Updated scenario'
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario: Updated scenario
			When something happens with someone
		"""

Rule: Step changes should be applied to the scenario

@tc:208
Scenario: Steps are synchronized back to feature file
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Scenario Outline: Sample scenario outline
			When <something> happens with <someone>

		Examples:
			| something | someone |
			| foo       | Joe     |
			| bar       | Jill    |
			| boz       | Jack    | 
		"""
	When the Test Case steps are updated to 
		| keyword | text                                          |
		| When    | [[something]] really happens with [[someone]] |
		| Then    | this is good for [[someone]]                  |
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario Outline: Sample scenario outline
			When <something> really happens with <someone>
			Then this is good for <someone>

		Examples:
			| something | someone |
			| foo       | Joe     |
			| bar       | Jill    |
			| boz       | Jack    | 
		"""

@tc:615
Scenario: Unchanged background steps are pulled
	The background steps are not included in the scenario
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Background:
			Given there is something

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case steps are updated to 
		| prefix      | keyword | text               |
		| Background: | Given   | there is something |
		|             | When    | I do something new |
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		Background:
			Given there is something

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something new
		"""

@tc:616
Scenario: Background steps changed in Azure DevOps
	The changed background steps are not included in the scenario, but 
	a warning is displayed
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Background:
			Given there is something

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case steps are updated to 
		| prefix      | keyword | text                       |
		| Background: | Given   | there is something changed |
		|             | When    | I do something new         |
	And SpecSync pull is executed
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		Background:
			Given there is something

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something new
		"""
	And the synchronization should finish with warnings
	And the log should contain "Background"
	And the log should contain "Given there is something changed"


Rule: Parameter value changes should be applied to the scenario outline

@tc:209
Scenario: Parameter values are synchronized back to Scenario Outline
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Scenario Outline: Sample scenario outline
			When <something> happens with <someone>

		Examples:
			| something | someone |
			| foo       | Joe     |
			| bar       | Jill    |
			| boz       | Jack    | 
		"""
	When the Test Case parameter data is updated to
		| something | someone |
		| one       | Tarzan  |
		| two       | Thomas  |
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario Outline: Sample scenario outline
			When <something> happens with <someone>

		Examples:
			| something | someone |
			| one       | Tarzan  |
			| two       | Thomas  |
		"""

Rule: Title changes should be applied to the scenario

@tc:210
Scenario: Title is synchronized back to feature file
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Scenario Outline: Sample scenario outline
			When <something> happens with <someone>

		Examples:
			| something | someone |
			| foo       | Joe     |
			| bar       | Jill    |
			| boz       | Jack    | 
		"""
	When the Test Case title should be updated to 'Scenario Outline: Updated scenario outline'
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario Outline: Updated scenario outline
			When <something> happens with <someone>

		Examples:
			| something | someone |
			| foo       | Joe     |
			| bar       | Jill    |
			| boz       | Jack    | 
		"""

Rule: Data table changes should be applied to the scenario

@tc:211
Scenario Outline: Data Table is synchronized back
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And the synchronizer is configured as
		| setting                                    | value                 |
		| synchronization/format/syncDataTableAsText | <syncDataTableAsText> |
	And there is an existing synchronized feature file
		"""
		Feature: Sample feature

		Scenario: Sample scenario
			When something happens with someone
				| something | someone |
				| foo       | Joe     |
				| bar       | Jill    |
				| boz       | Jack    | 
			But nothing more
		"""
	When the Test Case title should be updated to 'Scenario: Updated scenario'
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		"""
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario: Updated scenario
			When something happens with someone
				| something | someone |
				| foo       | Joe     |
				| bar       | Jill    |
				| boz       | Jack    | 
			But nothing more
		"""
Examples: 
	| description                     | syncDataTableAsText |
	| HTML table representation       | false               |
	| Plain-text table representation | true                |

Rule: Doc string changes should be applied to the scenario

@tc:212
Scenario: Doc String is synchronized back
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		```
		Feature: Sample feature

		Scenario: Sample scenario
			When something happens with someone
				"""
				something, someone
				  somewhere
				"""
			But nothing more
		```
	When the Test Case title should be updated to 'Scenario: Updated scenario'
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to:
		```
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario: Updated scenario
			When something happens with someone
				"""
				something, someone
				  somewhere
				"""
			But nothing more
		```

Rule: Tag changes should be applied to the scenario

@tc:617
Scenario: Tags are synchronized back to feature file
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @mytag @oldtag
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case tags are updated to "mytag, othertag"
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to contain:
		"""
		@tc:[id-of-test-case]
		@mytag @othertag
		Scenario: Sample scenario
			When I do something
		"""

@tc:618
Scenario: Some feature and rule tags changed in Azure DevOps
	Inherited tags (from feature or rule) are not re-added to the scenario, but 
	disappeared inherited tags are not removed from the parent headers but a warning is shown.
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And there is an existing synchronized feature file
		"""
		@unchanged_featuretag @removed_featuretag
		Feature: Sample feature

		@unchanged_ruletag @removed_ruletag
		Rule: My Rule

		@tc:[id-of-test-case]
		@mytag @oldtag
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case tags are updated to "unchanged_featuretag, unchanged_ruletag, mytag, othertag"
	And SpecSync pull is executed
	Then the feature file in the local repository should be updated to:
		"""
		@unchanged_featuretag @removed_featuretag
		Feature: Sample feature

		@unchanged_ruletag @removed_ruletag
		Rule: My Rule

		@tc:[id-of-test-case]
		@mytag @othertag
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronization should finish with warnings
	And the log should contain "@removed_featuretag"
	And the log should contain "@removed_ruletag"

Rule: Work item links are handled by pull command

@tc:826
Scenario: No changes in links: Existing link tag and existing link in Test Case
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                           | value |
		| synchronization/links[]/tagPrefix | bug   |
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case title should be updated to 'Scenario: Updated scenario'
	And the local repository is synchronized with pull
	Then the scenario should be updated to:
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Updated scenario
			When I do something
		"""

@tc:827
Scenario: Remote link removed: Existing link tag and no link in Test Case
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                           | value |
		| synchronization/links[]/tagPrefix | bug   |
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""
	When all links are removed from the Test Case
	And the local repository is synchronized with pull
	Then the scenario should be updated to:
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""

@tc:828
Scenario: Restore tracked link: New tracked link in Test Case
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                           | value |
		| synchronization/links[]/tagPrefix | bug   |
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""
	And the scenario has been updated to:
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case title should be updated to 'Scenario: Updated scenario'
	And SpecSync pull is executed with choosing "Remote" for conflict
	Then the scenario should be updated to:
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Updated scenario
			When I do something
		"""

@tc:829
Scenario: Pull link: New non-tracked link in Test Case
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                           | value |
		| synchronization/links[]/tagPrefix | bug   |
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the Test Case has got the Bug linked from the remote server
	When the local repository is synchronized with pull
	Then the scenario should be updated to:
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""

@tc:830
Scenario: The first matching link prefix is used for pulling links
	A warning is diplayed when multiple prefixes match
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                               | value                  |
		| synchronization/links[0]/tagPrefix    | parent                 |
		| synchronization/links[0]/relationship | Parent                 |
		| synchronization/links[1]/tagPrefix    | us                     |
		| synchronization/links[1]/relationship | [tests-reference-type] |
		| synchronization/links[1]/targetType   | User Story             |
		| synchronization/links[2]/tagPrefix    | bug                    |
		| synchronization/links[2]/relationship | [tests-reference-type] |
		| synchronization/links[3]/tagPrefix    | requirement            |
	And the synchronizer is configured to enable pull
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the Test Case has got the Bug linked from the remote server
	When SpecSync pull is executed
	Then the synchronization should finish with warnings
	And the scenario should be updated to:
		"""
		@tc:[id-of-test-case]
		@bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""

Rule: Not suppored local tags should not be removed on pull

@tc:1121
@customization @bypass-integration
Scenario: The scenario has a not supported local tag
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And the synchronizer is configured as
		| setting                                                       | value              |
		| customizations/ignoreNotSupportedLocalTags/enabled            | true               |
		| customizations/ignoreNotSupportedLocalTags/notSupportedTags[] | @not-supported-tag |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @mytag @oldtag @not-supported-tag
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case tags are updated to "mytag, othertag"
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to contain:
		"""
		@tc:[id-of-test-case]
		@mytag @not-supported-tag @othertag
		Scenario: Sample scenario
			When I do something
		"""


@tc:1122
@customization @bypass-integration
Scenario: The scenario has a not supported local tag that is also removed by fieldUpdates
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And the synchronizer is configured as
		| setting                                                       | value        |
		| customizations/ignoreNotSupportedLocalTags/enabled            | true         |
		| customizations/ignoreNotSupportedLocalTags/notSupportedTags[] | @designState |
		| synchronization/fieldUpdates/[state-field]/value              | Design       |
		| synchronization/fieldUpdates/[state-field]/condition          | @designState |

	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @atag @oldtag @designState
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case tags are updated to "atag, othertag"
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to contain:
		"""
		@tc:[id-of-test-case]
		@atag @designState @othertag
		Scenario: Sample scenario
			When I do something
		"""

Rule: Tags removed by field updates should not be removed on pull

@tc:1123
@bypass-integration
Scenario: The scenario has a tag that is removed by fieldUpdates
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And the synchronizer is configured as
		| setting                                              | value        |
		| synchronization/fieldUpdates/[state-field]/value     | Design       |
		| synchronization/fieldUpdates/[state-field]/condition | @designState |

	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @atag @oldtag @designState
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case tags are updated to "atag, othertag"
	And the local repository is synchronized with pull
	Then the feature file in the local repository should be updated to contain:
		"""
		@tc:[id-of-test-case]
		@atag @designState @othertag
		Scenario: Sample scenario
			When I do something
		"""

