﻿@customization @feature:1081
Feature: Do not synchronize title

Rule: The title of the Test Case should not be changed

@tc:1092
Scenario: Update scenario that is synchronized to a Test Case with custom title
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                      | value |
		| customizations/doNotSynchronizeTitle/enabled | true  |
	And there is a new Test Case as
		| field | value        |
		| title | Custom title |
	And there is a scenario in the local repository
		"""
		@tc:[id-of-test-case]
		Scenario: Updated sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then the Test Case title should be still "Custom title"

@tc:1093
@diag:hash
Scenario: Only the scenario title is updated
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                      | value |
		| customizations/doNotSynchronizeTitle/enabled | true  |
	And there is a new Test Case as
		| field | value        |
		| title | Custom title |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the scenario is updated and synchronized as:
		"""
		@tc:[id-of-test-case]
		Scenario: Updated sample scenario
			When I do something
		"""
	Then the scenario should be up-to-date

@tc:1094
@diag:hash
Scenario: Use another field to capture scenario title
	In this scenario, only the title change, but this still triggers the synchronization because it is used in field updates.
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                | value                     |
		| customizations/doNotSynchronizeTitle/enabled           | true                      |
		| synchronization/fieldUpdates/[description-field]/value | Scenario: {scenario-name} |
	And there is a new Test Case as
		| field | value        |
		| title | Custom title |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the scenario is updated and synchronized as:
		"""
		@tc:[id-of-test-case]
		Scenario: Updated sample scenario
			When I do something
		"""
	Then the Test Case title should be still "Custom title"
	And the Test Case fields should be set to the following values
		| field identifier    | value                             |
		| [description-field] | Scenario: Updated sample scenario |



Rule: The title should be set initially, but can be customized

@tc:1095
Scenario: The title is set initially
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                      | value |
		| customizations/doNotSynchronizeTitle/enabled | true  |
	And there is a scenario in the local repository
		"""
		Scenario: Sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then a new Test Case "Scenario: Sample scenario" should be created in the remote server

@tc:1096
Scenario: The inital title is customized
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                           | value        |
		| customizations/doNotSynchronizeTitle/enabled      | true         |
		| synchronization/fieldUpdates/[title-field]/value  | Custom title |
		| synchronization/fieldUpdates/[title-field]/update | onCreate     |
	And there is a scenario in the local repository
		"""
		Scenario: Sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then a new Test Case "Custom title" should be created in the remote server

Rule: Test Case changes can be pulled

@tc:1097
@adoSpecific @pull
Scenario: Create scenario from new Test Case
	Given there is a remote server project with an empty Test Suite 'MySuite'
	And the synchronizer is configured to enable pull
	And the synchronizer is configured to add Test Cases to Test Suite 'MySuite'
	And the synchronizer is configured to create local scenarios for new Test Cases
	And the synchronizer is configured as
		| setting                                      | value |
		| customizations/doNotSynchronizeTitle/enabled | true  |
	And there is a new Test Case in Test Suite 'MySuite' as
		| field | value                     |
		| title | Scenario: Sample scenario |
		| steps | When I do something       |
	When the local repository is synchronized with pull
	Then the local repository should contain a feature file '[id-of-test-case].feature' as
		"""
		Feature: [id-of-test-case]

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the feature file should be included in the local scope

@tc:1098
@pull
Scenario: Title changes are not pulled
	Given there is a remote server project
	And the synchronizer is configured to enable pull
	And the synchronizer is configured as
		| setting                                      | value |
		| customizations/doNotSynchronizeTitle/enabled | true  |
	And there is an existing synchronized scenario
		"""
		Scenario: Sample scenario
			When I do something
		"""
	When the Test Case title should be updated to 'Custom title'
	And the Test Case steps are updated to 
		| keyword | text                  |
		| When    | I really do something |
	And the local repository is synchronized with pull
	Then the scenario in the local repository should be updated to:
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I really do something
		"""
